@@ -35,7 +35,11 @@
 
 #include <assert.h>
 #include <iostream>
+#ifdef HAVE_ADT_OWNINGPTR_H
 #include <llvm/ADT/OwningPtr.h>
+#else
+#include <memory>
+#endif
 #include <llvm/Support/raw_ostream.h>
 #include <llvm/Support/CommandLine.h>
 #include <llvm/Support/Host.h>
@@ -71,6 +75,10 @@ using namespace std;
 using namespace clang;
 using namespace clang::driver;
 
+#ifdef HAVE_ADT_OWNINGPTR_H
+#define unique_ptr	llvm::OwningPtr
+#endif
+
 static llvm::cl::opt<string> InputFilename(llvm::cl::Positional,
 			llvm::cl::Required, llvm::cl::desc("<input file>"));
 static llvm::cl::list<string> Includes("I",
@@ -153,12 +161,17 @@ static Driver *construct_driver(const char *binary, DiagnosticsEngine &Diags)
 	return new Driver(binary, llvm::sys::getDefaultTargetTriple(),
 			    "", false, Diags);
 }
-#else
+#elif defined(DRIVER_CTOR_TAKES_DEFAULTIMAGENAME)
 static Driver *construct_driver(const char *binary, DiagnosticsEngine &Diags)
 {
 	return new Driver(binary, llvm::sys::getDefaultTargetTriple(),
 			    "", Diags);
 }
+#else
+static Driver *construct_driver(const char *binary, DiagnosticsEngine &Diags)
+{
+	return new Driver(binary, llvm::sys::getDefaultTargetTriple(), Diags);
+}
 #endif
 
 /* Create a CompilerInvocation object that stores the command line
@@ -170,11 +183,11 @@ static CompilerInvocation *construct_invocation(const char *filename,
 	DiagnosticsEngine &Diags)
 {
 	const char *binary = CLANG_PREFIX"/bin/clang";
-	const llvm::OwningPtr<Driver> driver(construct_driver(binary, Diags));
+	const unique_ptr<Driver> driver(construct_driver(binary, Diags));
 	std::vector<const char *> Argv;
 	Argv.push_back(binary);
 	Argv.push_back(filename);
-	const llvm::OwningPtr<Compilation> compilation(
+	const unique_ptr<Compilation> compilation(
 		driver->BuildCompilation(llvm::ArrayRef<const char *>(Argv)));
 	JobList &Jobs = compilation->getJobs();
 
@@ -218,7 +231,17 @@ static TextDiagnosticPrinter *construct_printer(void)
 
 #endif
 
-#ifdef CREATETARGETINFO_TAKES_POINTER
+#ifdef CREATETARGETINFO_TAKES_SHARED_PTR
+
+static TargetInfo *create_target_info(CompilerInstance *Clang,
+	DiagnosticsEngine &Diags)
+{
+	shared_ptr<TargetOptions> TO = Clang->getInvocation().TargetOpts;
+	TO->Triple = llvm::sys::getDefaultTargetTriple();
+	return TargetInfo::CreateTargetInfo(Diags, TO);
+}
+
+#elif defined(CREATETARGETINFO_TAKES_POINTER)
 
 static TargetInfo *create_target_info(CompilerInstance *Clang,
 	DiagnosticsEngine &Diags)
@@ -288,6 +311,23 @@ void add_path(HeaderSearchOptions &HSO, string Path)
 
 #endif
 
+#ifdef HAVE_SETMAINFILEID
+
+static void create_main_file_id(SourceManager &SM, const FileEntry *file)
+{
+	SM.setMainFileID(SM.createFileID(file, SourceLocation(),
+					SrcMgr::C_User));
+}
+
+#else
+
+static void create_main_file_id(SourceManager &SM, const FileEntry *file)
+{
+	SM.createMainFileID(file);
+}
+
+#endif
+
 int main(int argc, char *argv[])
 {
 	llvm::cl::ParseCommandLineOptions(argc, argv);
@@ -328,7 +368,7 @@ int main(int argc, char *argv[])
 
 	const FileEntry *file = Clang->getFileManager().getFile(InputFilename);
 	assert(file);
-	Clang->getSourceManager().createMainFileID(file);
+	create_main_file_id(Clang->getSourceManager(), file);
 
 	Clang->createASTContext();
 	MyASTConsumer consumer;
