@@ -465,9 +465,11 @@ error:
 	return NULL;
 }
 
-/* Create an expression representing the negation of "arg".
+/* Create an expression representing the unary operation "type" applied to
+ * "arg".
  */
-__isl_give isl_ast_expr *isl_ast_expr_neg(__isl_take isl_ast_expr *arg)
+__isl_give isl_ast_expr *isl_ast_expr_alloc_unary(enum isl_ast_op_type type,
+	__isl_take isl_ast_expr *arg)
 {
 	isl_ctx *ctx;
 	isl_ast_expr *expr = NULL;
@@ -476,7 +478,7 @@ __isl_give isl_ast_expr *isl_ast_expr_neg(__isl_take isl_ast_expr *arg)
 		return NULL;
 
 	ctx = isl_ast_expr_get_ctx(arg);
-	expr = isl_ast_expr_alloc_op(ctx, isl_ast_op_minus, 1);
+	expr = isl_ast_expr_alloc_op(ctx, type, 1);
 	if (!expr)
 		goto error;
 
@@ -488,6 +490,29 @@ error:
 	return NULL;
 }
 
+/* Create an expression representing the negation of "arg".
+ */
+__isl_give isl_ast_expr *isl_ast_expr_neg(__isl_take isl_ast_expr *arg)
+{
+	return isl_ast_expr_alloc_unary(isl_ast_op_minus, arg);
+}
+
+/* Create an expression representing the address of "expr".
+ */
+__isl_give isl_ast_expr *isl_ast_expr_address_of(__isl_take isl_ast_expr *expr)
+{
+	if (!expr)
+		return NULL;
+
+	if (isl_ast_expr_get_type(expr) != isl_ast_expr_op ||
+	    isl_ast_expr_get_op_type(expr) != isl_ast_op_access)
+		isl_die(isl_ast_expr_get_ctx(expr), isl_error_invalid,
+			"can only take address of access expressions",
+			return isl_ast_expr_free(expr));
+
+	return isl_ast_expr_alloc_unary(isl_ast_op_address_of, expr);
+}
+
 /* Create an expression representing the binary operation "type"
  * applied to "expr1" and "expr2".
  */
@@ -563,6 +588,46 @@ __isl_give isl_ast_expr *isl_ast_expr_or(__isl_take isl_ast_expr *expr1,
 	return isl_ast_expr_alloc_binary(isl_ast_op_or, expr1, expr2);
 }
 
+/* Create an expression representing "expr1" less than or equal to "expr2".
+ */
+__isl_give isl_ast_expr *isl_ast_expr_le(__isl_take isl_ast_expr *expr1,
+	__isl_take isl_ast_expr *expr2)
+{
+	return isl_ast_expr_alloc_binary(isl_ast_op_le, expr1, expr2);
+}
+
+/* Create an expression representing "expr1" less than "expr2".
+ */
+__isl_give isl_ast_expr *isl_ast_expr_lt(__isl_take isl_ast_expr *expr1,
+	__isl_take isl_ast_expr *expr2)
+{
+	return isl_ast_expr_alloc_binary(isl_ast_op_lt, expr1, expr2);
+}
+
+/* Create an expression representing "expr1" greater than or equal to "expr2".
+ */
+__isl_give isl_ast_expr *isl_ast_expr_ge(__isl_take isl_ast_expr *expr1,
+	__isl_take isl_ast_expr *expr2)
+{
+	return isl_ast_expr_alloc_binary(isl_ast_op_ge, expr1, expr2);
+}
+
+/* Create an expression representing "expr1" greater than "expr2".
+ */
+__isl_give isl_ast_expr *isl_ast_expr_gt(__isl_take isl_ast_expr *expr1,
+	__isl_take isl_ast_expr *expr2)
+{
+	return isl_ast_expr_alloc_binary(isl_ast_op_gt, expr1, expr2);
+}
+
+/* Create an expression representing "expr1" equal to "expr2".
+ */
+__isl_give isl_ast_expr *isl_ast_expr_eq(__isl_take isl_ast_expr *expr1,
+	__isl_take isl_ast_expr *expr2)
+{
+	return isl_ast_expr_alloc_binary(isl_ast_op_eq, expr1, expr2);
+}
+
 /* Create an expression representing an access to "array" with index
  * expressions "indices".
  */
@@ -1102,6 +1167,9 @@ __isl_give isl_ast_expr *isl_ast_node_user_get_expr(
 {
 	if (!node)
 		return NULL;
+	if (node->type != isl_ast_node_user)
+		isl_die(isl_ast_node_get_ctx(node), isl_error_invalid,
+			"not a user node", return NULL);
 
 	return isl_ast_expr_copy(node->u.e.expr);
 }
@@ -1150,7 +1218,8 @@ static char *op_str[] = {
 	[isl_ast_op_ge] = ">=",
 	[isl_ast_op_lt] = "<",
 	[isl_ast_op_gt] = ">",
-	[isl_ast_op_member] = "."
+	[isl_ast_op_member] = ".",
+	[isl_ast_op_address_of] = "&"
 };
 
 /* Precedence in C of the various operators.
@@ -1181,7 +1250,8 @@ static int op_prec[] = {
 	[isl_ast_op_gt] = 8,
 	[isl_ast_op_call] = 2,
 	[isl_ast_op_access] = 2,
-	[isl_ast_op_member] = 2
+	[isl_ast_op_member] = 2,
+	[isl_ast_op_address_of] = 3
 };
 
 /* Is the operator left-to-right associative?
@@ -1210,7 +1280,8 @@ static int op_left[] = {
 	[isl_ast_op_gt] = 1,
 	[isl_ast_op_call] = 1,
 	[isl_ast_op_access] = 1,
-	[isl_ast_op_member] = 1
+	[isl_ast_op_member] = 1,
+	[isl_ast_op_address_of] = 0
 };
 
 static int is_and(enum isl_ast_op_type op)
@@ -1492,16 +1563,21 @@ static __isl_give isl_printer *print_ast_node_isl(__isl_take isl_printer *p,
  * as well.
  * If the node is an if node with an else, then we print a block
  * to avoid spurious dangling else warnings emitted by some compilers.
+ * If the ast_always_print_block option has been set, then we print a block.
  */
 static int need_block(__isl_keep isl_ast_node *node)
 {
+	isl_ctx *ctx;
+
 	if (node->type == isl_ast_node_block)
 		return 1;
 	if (node->type == isl_ast_node_for && node->u.f.degenerate)
 		return 1;
 	if (node->type == isl_ast_node_if && node->u.i.else_node)
 		return 1;
-	return 0;
+
+	ctx = isl_ast_node_get_ctx(node);
+	return isl_options_get_ast_always_print_block(ctx);
 }
 
 static __isl_give isl_printer *print_ast_node_c(__isl_take isl_printer *p,
