@@ -1,10 +1,13 @@
 /*
  * Copyright 2012      Ecole Normale Superieure
+ * Copyright 2014      INRIA Rocquencourt
  *
  * Use of this software is governed by the MIT license
  *
  * Written by Sven Verdoolaege,
  * Ecole Normale Superieure, 45 rue d’Ulm, 75230 Paris, France
+ * and Inria Paris - Rocquencourt, Domaine de Voluceau - Rocquencourt,
+ * B.P. 105 - 78153 Le Chesnay, France
  */
 
 #include <isl/map.h>
@@ -98,7 +101,7 @@ static __isl_give isl_id *generate_name(isl_ctx *ctx, int i,
  *
  * The input set is usually a parameter domain, but we currently allow it to
  * be any kind of set.  We set the domain of the returned isl_ast_build
- * to "set" and initialize all the other field to default values.
+ * to "set" and initialize all the other fields to default values.
  */
 __isl_give isl_ast_build *isl_ast_build_from_context(__isl_take isl_set *set)
 {
@@ -628,7 +631,24 @@ static int extract_single_piece(__isl_take isl_set *set,
 	return -1;
 }
 
-/* Check if the given bounds on the current dimension imply that
+/* Intersect "set" with the stride constraint of "build", if any.
+ */
+static __isl_give isl_set *intersect_stride_constraint(__isl_take isl_set *set,
+	__isl_keep isl_ast_build *build)
+{
+	isl_set *stride;
+
+	if (!build)
+		return isl_set_free(set);
+	if (!isl_ast_build_has_stride(build, build->depth))
+		return set;
+
+	stride = isl_ast_build_get_stride_constraint(build);
+	return isl_set_intersect(set, stride);
+}
+
+/* Check if the given bounds on the current dimension (together with
+ * the stride constraint, if any) imply that
  * this current dimension attains only a single value (in terms of
  * parameters and outer dimensions).
  * If so, we record it in build->value.
@@ -655,6 +675,7 @@ static __isl_give isl_ast_build *update_values(
 
 	set = isl_set_from_basic_set(bounds);
 	set = isl_set_intersect(set, isl_set_copy(build->domain));
+	set = intersect_stride_constraint(set, build);
 	it_map = isl_ast_build_map_to_iterator(build, set);
 
 	sv = isl_map_is_single_valued(it_map);
@@ -687,7 +708,7 @@ static __isl_give isl_ast_build *update_values(
 }
 
 /* Update the AST build based on the given loop bounds for
- * the current dimension.
+ * the current dimension and the stride information available in the build.
  *
  * We first make sure that the bounds do not refer to any iterators
  * that have already been eliminated.
@@ -700,10 +721,9 @@ static __isl_give isl_ast_build *update_values(
  * be defined on a subset of the domain.  Plugging in the value
  * would restrict the build domain to this subset, while this
  * restriction may not be reflected in the generated code.
- * build->domain may, however, already refer to the current dimension
- * due an earlier call to isl_ast_build_include_stride.  If so, we need
- * to eliminate the dimension so that we do not introduce it in any other sets.
  * Finally, we intersect build->domain with the updated bounds.
+ * We also add the stride constraint unless we have been able
+ * to find a fixed value expressed as a single affine expression.
  *
  * Note that the check for a fixed value in update_values requires
  * us to intersect the bounds with the current build domain.
@@ -734,11 +754,7 @@ __isl_give isl_ast_build *isl_ast_build_set_loop_bounds(
 		set = isl_set_compute_divs(set);
 		build->pending = isl_set_intersect(build->pending,
 							isl_set_copy(set));
-		if (isl_ast_build_has_stride(build, build->depth)) {
-			build->domain = isl_set_eliminate(build->domain,
-						isl_dim_set, build->depth, 1);
-			build->domain = isl_set_compute_divs(build->domain);
-		}
+		build->domain = isl_set_intersect(build->domain, set);
 	} else {
 		isl_basic_set *generated, *pending;
 
@@ -752,10 +768,13 @@ __isl_give isl_ast_build *isl_ast_build_set_loop_bounds(
 				    generated, isl_dim_set, build->depth, 1);
 		build->generated = isl_set_intersect(build->generated,
 					isl_set_from_basic_set(generated));
+		build->domain = isl_set_intersect(build->domain, set);
+		build = isl_ast_build_include_stride(build);
+		if (!build)
+			goto error;
 	}
 	isl_basic_set_free(bounds);
 
-	build->domain = isl_set_intersect(build->domain, set);
 	if (!build->domain || !build->pending || !build->generated)
 		return isl_ast_build_free(build);
 
@@ -766,38 +785,6 @@ error:
 	return NULL;
 }
 
-/* Update build->domain based on the constraints enforced by inner loops.
- *
- * The constraints in build->pending may end up not getting generated
- * if they are implied by "enforced".  We therefore reconstruct
- * build->domain from build->generated and build->pending, dropping
- * those constraint in build->pending that may not get generated.
- */
-__isl_give isl_ast_build *isl_ast_build_set_enforced(
-	__isl_take isl_ast_build *build, __isl_take isl_basic_set *enforced)
-{
-	isl_set *set;
-
-	build = isl_ast_build_cow(build);
-	if (!build)
-		goto error;
-
-	set = isl_set_from_basic_set(enforced);
-	set = isl_set_gist(isl_set_copy(build->pending), set);
-	set = isl_set_intersect(isl_set_copy(build->generated), set);
-
-	isl_set_free(build->domain);
-	build->domain = set;
-
-	if (!build->domain)
-		return isl_ast_build_free(build);
-
-	return build;
-error:
-	isl_basic_set_free(enforced);
-	return isl_ast_build_free(build);
-}
-
 /* Intersect build->domain with "set", where "set" is specified
  * in terms of the internal schedule domain.
  */
@@ -847,6 +834,30 @@ error:
 	return NULL;
 }
 
+/* Replace the set of pending constraints by "guard", which is then
+ * no longer considered as pending.
+ * That is, add "guard" to the generated constraints and clear all pending
+ * constraints, making the domain equal to the generated constraints.
+ */
+__isl_give isl_ast_build *isl_ast_build_replace_pending_by_guard(
+	__isl_take isl_ast_build *build, __isl_take isl_set *guard)
+{
+	build = isl_ast_build_restrict_generated(build, guard);
+	build = isl_ast_build_cow(build);
+	if (!build)
+		return NULL;
+
+	isl_set_free(build->domain);
+	build->domain = isl_set_copy(build->generated);
+	isl_set_free(build->pending);
+	build->pending = isl_set_universe(isl_set_get_space(build->domain));
+
+	if (!build->pending)
+		return isl_ast_build_free(build);
+
+	return build;
+}
+
 /* Intersect build->pending and build->domain with "set",
  * where "set" is specified in terms of the internal schedule domain.
  */
@@ -915,6 +926,33 @@ __isl_give isl_set *isl_ast_build_get_domain(__isl_keep isl_ast_build *build)
 	return build ? isl_set_copy(build->domain) : NULL;
 }
 
+/* Return a copy of the set of pending constraints.
+ */
+__isl_give isl_set *isl_ast_build_get_pending(
+	__isl_keep isl_ast_build *build)
+{
+	return build ? isl_set_copy(build->pending) : NULL;
+}
+
+/* Return a copy of the set of generated constraints.
+ */
+__isl_give isl_set *isl_ast_build_get_generated(
+	__isl_keep isl_ast_build *build)
+{
+	return build ? isl_set_copy(build->generated) : NULL;
+}
+
+/* Return the number of variables of the given type
+ * in the (internal) schedule space.
+ */
+unsigned isl_ast_build_dim(__isl_keep isl_ast_build *build,
+	enum isl_dim_type type)
+{
+	if (!build)
+		return 0;
+	return isl_set_dim(build->domain, type);
+}
+
 /* Return the (schedule) space of "build".
  *
  * If "internal" is set, then this space is the space of the internal
@@ -1251,7 +1289,7 @@ static int detect_stride(__isl_take isl_constraint *c, void *user)
 	n_div = isl_constraint_dim(c, isl_dim_div);
 	for (i = 0; i < n_div; ++i) {
 		v = isl_constraint_get_coefficient_val(c, isl_dim_div, i);
-		v = isl_val_gcd(stride, v);
+		stride = isl_val_gcd(stride, v);
 	}
 
 	v = isl_constraint_get_coefficient_val(c, isl_dim_set, data->pos);
@@ -1866,6 +1904,24 @@ error:
 	return NULL;
 }
 
+/* Include information about what we know about the iterators of
+ * already generated loops to "set".
+ *
+ * We currently only plug in the known affine values of outer loop
+ * iterators.
+ * In principle we could also introduce equalities or even other
+ * constraints implied by the intersection of "set" and build->domain.
+ */
+__isl_give isl_set *isl_ast_build_specialize(__isl_keep isl_ast_build *build,
+	__isl_take isl_set *set)
+{
+	if (!build)
+		return isl_set_free(set);
+
+	return isl_set_preimage_multi_aff(set,
+					isl_multi_aff_copy(build->values));
+}
+
 /* Simplify the map "map" based on what we know about
  * the iterators of already generated loops.
  *
