@@ -1,10 +1,13 @@
 /*
  * Copyright 2012      Ecole Normale Superieure
+ * Copyright 2014      INRIA Rocquencourt
  *
  * Use of this software is governed by the MIT license
  *
  * Written by Sven Verdoolaege,
  * Ecole Normale Superieure, 45 rue d’Ulm, 75230 Paris, France
+ * and Inria Paris - Rocquencourt, Domaine de Voluceau - Rocquencourt,
+ * B.P. 105 - 78153 Le Chesnay, France
  */
 
 #include <isl_ast_private.h>
@@ -135,26 +138,61 @@ static int equal_independent_guards(__isl_keep isl_ast_graft_list *list,
 	return equal;
 }
 
+/* Hoist "guard" out of the current level (given by "build").
+ *
+ * In particular, eliminate the dimension corresponding to the current depth.
+ */
+static __isl_give isl_set *hoist_guard(__isl_take isl_set *guard,
+	__isl_keep isl_ast_build *build)
+{
+	int depth;
+
+	depth = isl_ast_build_get_depth(build);
+	if (depth < isl_set_dim(guard, isl_dim_set)) {
+		guard = isl_set_remove_divs_involving_dims(guard,
+						isl_dim_set, depth, 1);
+		guard = isl_set_eliminate(guard, isl_dim_set, depth, 1);
+		guard = isl_set_compute_divs(guard);
+	}
+
+	return guard;
+}
+
 /* Extract a common guard from the grafts in "list" that can be hoisted
  * out of the current level.  If no such guard can be found, then return
  * a universal set.
  *
  * If all the grafts in the list have the same guard and if this guard
  * is independent of the current level, then it can be hoisted out.
+ * If there is only one graft in the list and if its guard
+ * depends on the current level, then we eliminate this level and
+ * return the result.
+ *
  * Otherwise, we return the unshifted simple hull of the guards.
+ * In order to be able to hoist as many constraints as possible,
+ * but at the same time avoid hoisting constraints that did not
+ * appear in the guards in the first place, we intersect the guards
+ * with all the information that is available (i.e., the domain
+ * from the build and the enforced constraints of the graft) and
+ * compute the unshifted hull of the result using only constraints
+ * from the original guards.
+ * In particular, intersecting the guards with other known information
+ * allows us to hoist guards that are only explicit is some of
+ * the grafts and implicit in the others.
  *
  * The special case for equal guards is needed in case those guards
  * are non-convex.  Taking the simple hull would remove information
  * and would not allow for these guards to be hoisted completely.
  */
-static __isl_give isl_set *extract_hoistable_guard(
+__isl_give isl_set *isl_ast_graft_list_extract_hoistable_guard(
 	__isl_keep isl_ast_graft_list *list, __isl_keep isl_ast_build *build)
 {
 	int i, n;
-	int depth;
-	isl_ast_graft *graft_0;
 	int equal;
+	isl_ctx *ctx;
 	isl_set *guard;
+	isl_set_list *set_list;
+	isl_basic_set *hull;
 
 	if (!list || !build)
 		return NULL;
@@ -167,45 +205,41 @@ static __isl_give isl_set *extract_hoistable_guard(
 	if (equal < 0)
 		return NULL;
 
-	graft_0 = isl_ast_graft_list_get_ast_graft(list, 0);
-	if (!graft_0)
-		return NULL;
-	guard = isl_set_copy(graft_0->guard);
-	isl_ast_graft_free(graft_0);
-	if (equal)
-		return guard;
+	if (equal || n == 1) {
+		isl_ast_graft *graft_0;
 
-	depth = isl_ast_build_get_depth(build);
-	if (depth < isl_set_dim(guard, isl_dim_set)) {
-		guard = isl_set_remove_divs_involving_dims(guard,
-						isl_dim_set, depth, 1);
-		guard = isl_set_eliminate(guard, isl_dim_set, depth, 1);
-		guard = isl_set_compute_divs(guard);
+		graft_0 = isl_ast_graft_list_get_ast_graft(list, 0);
+		if (!graft_0)
+			return NULL;
+		guard = isl_set_copy(graft_0->guard);
+		if (!equal)
+			guard = hoist_guard(guard, build);
+		isl_ast_graft_free(graft_0);
+		return guard;
 	}
 
-	for (i = 1; i < n; ++i) {
+	ctx = isl_ast_build_get_ctx(build);
+	set_list = isl_set_list_alloc(ctx, n);
+	guard = isl_set_empty(isl_ast_build_get_space(build, 1));
+	for (i = 0; i < n; ++i) {
 		isl_ast_graft *graft;
-		isl_basic_set *hull;
-		int is_universe;
-
-		is_universe = isl_set_plain_is_universe(guard);
-		if (is_universe < 0)
-			guard = isl_set_free(guard);
-		if (is_universe)
-			break;
+		isl_basic_set *enforced;
+		isl_set *guard_i;
 
 		graft = isl_ast_graft_list_get_ast_graft(list, i);
-		if (!graft) {
-			guard = isl_set_free(guard);
-			break;
-		}
-		guard = isl_set_union(guard, isl_set_copy(graft->guard));
-		hull = isl_set_unshifted_simple_hull(guard);
-		guard = isl_set_from_basic_set(hull);
+		enforced = isl_ast_graft_get_enforced(graft);
+		guard_i = isl_set_copy(graft->guard);
 		isl_ast_graft_free(graft);
+		set_list = isl_set_list_add(set_list, isl_set_copy(guard_i));
+		guard_i = isl_set_intersect(guard_i,
+					    isl_set_from_basic_set(enforced));
+		guard_i = isl_set_intersect(guard_i,
+					    isl_ast_build_get_domain(build));
+		guard = isl_set_union(guard, guard_i);
 	}
-
-	return guard;
+	hull = isl_set_unshifted_simple_hull_from_set_list(guard, set_list);
+	guard = isl_set_from_basic_set(hull);
+	return hoist_guard(guard, build);
 }
 
 /* Internal data structure used inside insert_if.
@@ -301,8 +335,6 @@ static __isl_give isl_ast_graft *insert_if_node(
 	}
 
 	build = isl_ast_build_copy(build);
-	build = isl_ast_build_set_enforced(build,
-					isl_ast_graft_get_enforced(graft));
 	graft->node = ast_node_insert_if(graft->node, guard, build);
 	isl_ast_build_free(build);
 
@@ -649,7 +681,7 @@ static __isl_give isl_ast_node_list *extract_node_list(
  *
  * We assume that the number of children is at least one.
  */
-static __isl_give isl_basic_set *extract_shared_enforced(
+__isl_give isl_basic_set *isl_ast_graft_list_extract_shared_enforced(
 	__isl_keep isl_ast_graft_list *list,
 	__isl_keep isl_ast_build *build)
 {
@@ -685,7 +717,9 @@ static __isl_give isl_basic_set *extract_shared_enforced(
 /* Record "guard" in "graft" so that it will be enforced somewhere
  * up the tree.  If the graft already has a guard, then it may be partially
  * redundant in combination with the new guard and in the context
- * of build->domain.  We therefore (re)compute the gist of the intersection
+ * the generated constraints of "build".  In fact, the new guard
+ * may in itself have some redundant constraints.
+ * We therefore (re)compute the gist of the intersection
  * and coalesce the result.
  */
 static __isl_give isl_ast_graft *store_guard(__isl_take isl_ast_graft *graft,
@@ -705,7 +739,8 @@ static __isl_give isl_ast_graft *store_guard(__isl_take isl_ast_graft *graft,
 	}
 
 	graft->guard = isl_set_intersect(graft->guard, guard);
-	graft->guard = isl_ast_build_compute_gist(build, graft->guard);
+	graft->guard = isl_set_gist(graft->guard,
+				    isl_ast_build_get_generated(build));
 	graft->guard = isl_set_coalesce(graft->guard);
 	if (!graft->guard)
 		return isl_ast_graft_free(graft);
@@ -746,12 +781,57 @@ static __isl_give isl_ast_graft_list *gist_guards(
 	return list;
 }
 
-/* Combine the grafts in the list into a single graft.
+/* For each graft in "list", replace its guard with the gist with
+ * respect to "context".
+ */
+__isl_give isl_ast_graft_list *isl_ast_graft_list_gist_guards(
+	__isl_take isl_ast_graft_list *list, __isl_take isl_set *context)
+{
+	list = gist_guards(list, context);
+	isl_set_free(context);
+
+	return list;
+}
+
+/* Allocate a graft in "build" based on the list of grafts in "sub_build".
+ * "guard" and "enforced" are the guard and enforced constraints
+ * of the allocated graft.  The guard is used to simplify the guards
+ * of the elements in "list".
  *
- * If "up" is set then the resulting graft will be used at an outer level.
- * In this case, "build" refers to the outer level, while "sub_build"
- * refers to the inner level.  If "up" is not set, then the same build
- * should be passed to both arguments.
+ * The node is initialized to either a block containing the nodes of "children"
+ * or, if there is only a single child, the node of that child.
+ * If the current level requires a for node, it should be inserted by
+ * a subsequent call to isl_ast_graft_insert_for.
+ */
+__isl_give isl_ast_graft *isl_ast_graft_alloc_from_children(
+	__isl_take isl_ast_graft_list *list, __isl_take isl_set *guard,
+	__isl_take isl_basic_set *enforced, __isl_keep isl_ast_build *build,
+	__isl_keep isl_ast_build *sub_build)
+{
+	isl_ast_build *guard_build;
+	isl_ast_node *node;
+	isl_ast_node_list *node_list;
+	isl_ast_graft *graft;
+
+	guard_build = isl_ast_build_copy(sub_build);
+	guard_build = isl_ast_build_replace_pending_by_guard(guard_build,
+						isl_set_copy(guard));
+	list = gist_guards(list, guard);
+	list = insert_pending_guard_nodes(list, guard_build);
+	isl_ast_build_free(guard_build);
+
+	node_list = extract_node_list(list);
+	node = isl_ast_node_from_ast_node_list(node_list);
+	isl_ast_graft_list_free(list);
+
+	graft = isl_ast_graft_alloc(node, build);
+	graft = store_guard(graft, guard, build);
+	graft = isl_ast_graft_enforce(graft, enforced);
+
+	return graft;
+}
+
+/* Combine the grafts in the list into a single graft.
  *
  * The guard is initialized to the shared guard of the list elements (if any),
  * provided it does not depend on the current dimension.
@@ -767,37 +847,20 @@ static __isl_give isl_ast_graft_list *gist_guards(
  * or, if there is only a single element, the node of that element.
  */
 static __isl_give isl_ast_graft *ast_graft_list_fuse(
-	__isl_take isl_ast_graft_list *list, __isl_keep isl_ast_build *build,
-	__isl_keep isl_ast_build *sub_build, int up)
+	__isl_take isl_ast_graft_list *list, __isl_keep isl_ast_build *build)
 {
-	isl_ctx *ctx;
-	isl_ast_node *node;
 	isl_ast_graft *graft;
-	isl_ast_node_list *node_list;
+	isl_basic_set *enforced;
 	isl_set *guard;
 
 	if (!list)
 		return NULL;
 
-	ctx = isl_ast_build_get_ctx(build);
-	guard = extract_hoistable_guard(list, build);
-	list = gist_guards(list, guard);
-	list = insert_pending_guard_nodes(list, sub_build);
+	enforced = isl_ast_graft_list_extract_shared_enforced(list, build);
+	guard = isl_ast_graft_list_extract_hoistable_guard(list, build);
+	graft = isl_ast_graft_alloc_from_children(list, guard, enforced,
+						    build, build);
 
-	node_list = extract_node_list(list);
-	node = isl_ast_node_from_ast_node_list(node_list);
-
-	graft = isl_ast_graft_alloc(node, build);
-
-	if (!up || isl_options_get_ast_build_exploit_nested_bounds(ctx)) {
-		isl_basic_set *enforced;
-		enforced = extract_shared_enforced(list, build);
-		graft = isl_ast_graft_enforce(graft, enforced);
-	}
-
-	graft = store_guard(graft, guard, build);
-
-	isl_ast_graft_list_free(list);
 	return graft;
 }
 
@@ -815,7 +878,7 @@ __isl_give isl_ast_graft_list *isl_ast_graft_list_fuse(
 		return NULL;
 	if (isl_ast_graft_list_n_ast_graft(list) <= 1)
 		return list;
-	graft = ast_graft_list_fuse(list, build, build, 0);
+	graft = ast_graft_list_fuse(list, build);
 	return isl_ast_graft_list_from_ast_graft(graft);
 }
 
@@ -835,24 +898,7 @@ static __isl_give isl_ast_graft *isl_ast_graft_fuse(
 	list = isl_ast_graft_list_add(list, graft1);
 	list = isl_ast_graft_list_add(list, graft2);
 
-	return ast_graft_list_fuse(list, build, build, 0);
-}
-
-/* Allocate a graft for the current level based on the list of grafts
- * of the inner level.
- * "build" represents the context of the current level.
- * "sub_build" represents the context of the inner level.
- *
- * The node is initialized to either a block containing the nodes of "children"
- * or, if there is only a single child, the node of that child.
- * If the current level requires a for node, it should be inserted by
- * a subsequent call to isl_ast_graft_insert_for.
- */
-__isl_give isl_ast_graft *isl_ast_graft_alloc_level(
-	__isl_take isl_ast_graft_list *children,
-	__isl_keep isl_ast_build *build, __isl_keep isl_ast_build *sub_build)
-{
-	return ast_graft_list_fuse(children, build, sub_build, 1);
+	return ast_graft_list_fuse(list, build);
 }
 
 /* Insert a for node enclosing the current graft->node.
@@ -942,30 +988,12 @@ __isl_give isl_set *isl_ast_graft_get_guard(__isl_keep isl_ast_graft *graft)
 }
 
 /* Record that "guard" needs to be inserted in "graft".
- *
- * We first simplify the guard in the context of the enforced set and
- * then we store the guard in case we may be able
- * to hoist it to higher levels and/or combine it with those of other grafts.
  */
 __isl_give isl_ast_graft *isl_ast_graft_add_guard(
 	__isl_take isl_ast_graft *graft,
 	__isl_take isl_set *guard, __isl_keep isl_ast_build *build)
 {
-	isl_basic_set *enforced;
-
-	if (!graft || !build)
-		goto error;
-
-	enforced = isl_basic_set_copy(graft->enforced);
-	guard = isl_set_gist(guard, isl_set_from_basic_set(enforced));
-
-	graft = store_guard(graft, guard, build);
-
-	return graft;
-error:
-	isl_set_free(guard);
-	isl_ast_graft_free(graft);
-	return NULL;
+	return store_guard(graft, guard, build);
 }
 
 /* Reformulate the "graft", which was generated in the context
