@@ -789,6 +789,8 @@ static __isl_give isl_pw_aff *accept_extended_affine(struct isl_stream *s,
 	cond = isl_map_universe(isl_space_unwrap(space));
 
 	if (push_aff(s, line, col, pwaff) < 0)
+		cond = isl_map_free(cond);
+	if (!cond)
 		return NULL;
 
 	cond = read_formula(s, v, cond, rational);
@@ -2551,6 +2553,9 @@ static __isl_give isl_basic_map *basic_map_read(struct isl_stream *s)
 	struct isl_basic_map *bmap;
 
 	obj = obj_read(s);
+	if (obj.v && (obj.type != isl_obj_map && obj.type != isl_obj_set))
+		isl_die(s->ctx, isl_error_invalid, "not a (basic) set or map",
+			goto error);
 	map = obj.v;
 	if (!map)
 		return NULL;
@@ -2569,7 +2574,7 @@ static __isl_give isl_basic_map *basic_map_read(struct isl_stream *s)
 
 	return bmap;
 error:
-	isl_map_free(map);
+	obj.type->free(obj.v);
 	return NULL;
 }
 
@@ -3107,7 +3112,7 @@ error:
 }
 
 /* Read a multi-affine expression from "s".
- * If the multi-affine expression has a domain, then then tuple
+ * If the multi-affine expression has a domain, then the tuple
  * representing this domain cannot involve any affine expressions.
  * The tuple representing the actual expressions needs to consist
  * of only affine expressions.  Moreover, these expressions can
