@@ -1,7 +1,7 @@
 /*
  * Copyright 2008-2009 Katholieke Universiteit Leuven
  *
- * Use of this software is governed by the GNU LGPLv2.1 license
+ * Use of this software is governed by the MIT license
  *
  * Written by Sven Verdoolaege, K.U.Leuven, Departement
  * Computerwetenschappen, Celestijnenlaan 200A, B-3001 Leuven, Belgium
@@ -10,16 +10,16 @@
 #include <isl_ctx_private.h>
 #include <isl_map_private.h>
 #include "isl_sample.h"
-#include "isl_sample_piplib.h"
 #include <isl/vec.h>
 #include <isl/mat.h>
-#include <isl/seq.h>
+#include <isl_seq.h>
 #include "isl_equalities.h"
 #include "isl_tab.h"
 #include "isl_basis_reduction.h"
 #include <isl_factorization.h>
 #include <isl_point_private.h>
 #include <isl_options_private.h>
+#include <isl_vec_private.h>
 
 static struct isl_vec *empty_sample(struct isl_basic_set *bset)
 {
@@ -109,146 +109,6 @@ error:
 	return NULL;
 }
 
-static struct isl_mat *independent_bounds(struct isl_basic_set *bset)
-{
-	int i, j, n;
-	struct isl_mat *dirs = NULL;
-	struct isl_mat *bounds = NULL;
-	unsigned dim;
-
-	if (!bset)
-		return NULL;
-
-	dim = isl_basic_set_n_dim(bset);
-	bounds = isl_mat_alloc(bset->ctx, 1+dim, 1+dim);
-	if (!bounds)
-		return NULL;
-
-	isl_int_set_si(bounds->row[0][0], 1);
-	isl_seq_clr(bounds->row[0]+1, dim);
-	bounds->n_row = 1;
-
-	if (bset->n_ineq == 0)
-		return bounds;
-
-	dirs = isl_mat_alloc(bset->ctx, dim, dim);
-	if (!dirs) {
-		isl_mat_free(bounds);
-		return NULL;
-	}
-	isl_seq_cpy(dirs->row[0], bset->ineq[0]+1, dirs->n_col);
-	isl_seq_cpy(bounds->row[1], bset->ineq[0], bounds->n_col);
-	for (j = 1, n = 1; n < dim && j < bset->n_ineq; ++j) {
-		int pos;
-
-		isl_seq_cpy(dirs->row[n], bset->ineq[j]+1, dirs->n_col);
-
-		pos = isl_seq_first_non_zero(dirs->row[n], dirs->n_col);
-		if (pos < 0)
-			continue;
-		for (i = 0; i < n; ++i) {
-			int pos_i;
-			pos_i = isl_seq_first_non_zero(dirs->row[i], dirs->n_col);
-			if (pos_i < pos)
-				continue;
-			if (pos_i > pos)
-				break;
-			isl_seq_elim(dirs->row[n], dirs->row[i], pos,
-					dirs->n_col, NULL);
-			pos = isl_seq_first_non_zero(dirs->row[n], dirs->n_col);
-			if (pos < 0)
-				break;
-		}
-		if (pos < 0)
-			continue;
-		if (i < n) {
-			int k;
-			isl_int *t = dirs->row[n];
-			for (k = n; k > i; --k)
-				dirs->row[k] = dirs->row[k-1];
-			dirs->row[i] = t;
-		}
-		++n;
-		isl_seq_cpy(bounds->row[n], bset->ineq[j], bounds->n_col);
-	}
-	isl_mat_free(dirs);
-	bounds->n_row = 1+n;
-	return bounds;
-}
-
-static void swap_inequality(struct isl_basic_set *bset, int a, int b)
-{
-	isl_int *t = bset->ineq[a];
-	bset->ineq[a] = bset->ineq[b];
-	bset->ineq[b] = t;
-}
-
-/* Skew into positive orthant and project out lineality space.
- *
- * We perform a unimodular transformation that turns a selected
- * maximal set of linearly independent bounds into constraints
- * on the first dimensions that impose that these first dimensions
- * are non-negative.  In particular, the constraint matrix is lower
- * triangular with positive entries on the diagonal and negative
- * entries below.
- * If "bset" has a lineality space then these constraints (and therefore
- * all constraints in bset) only involve the first dimensions.
- * The remaining dimensions then do not appear in any constraints and
- * we can select any value for them, say zero.  We therefore project
- * out this final dimensions and plug in the value zero later.  This
- * is accomplished by simply dropping the final columns of
- * the unimodular transformation.
- */
-static struct isl_basic_set *isl_basic_set_skew_to_positive_orthant(
-	struct isl_basic_set *bset, struct isl_mat **T)
-{
-	struct isl_mat *U = NULL;
-	struct isl_mat *bounds = NULL;
-	int i, j;
-	unsigned old_dim, new_dim;
-
-	*T = NULL;
-	if (!bset)
-		return NULL;
-
-	isl_assert(bset->ctx, isl_basic_set_n_param(bset) == 0, goto error);
-	isl_assert(bset->ctx, bset->n_div == 0, goto error);
-	isl_assert(bset->ctx, bset->n_eq == 0, goto error);
-	
-	old_dim = isl_basic_set_n_dim(bset);
-	/* Try to move (multiples of) unit rows up. */
-	for (i = 0, j = 0; i < bset->n_ineq; ++i) {
-		int pos = isl_seq_first_non_zero(bset->ineq[i]+1, old_dim);
-		if (pos < 0)
-			continue;
-		if (isl_seq_first_non_zero(bset->ineq[i]+1+pos+1,
-						old_dim-pos-1) >= 0)
-			continue;
-		if (i != j)
-			swap_inequality(bset, i, j);
-		++j;
-	}
-	bounds = independent_bounds(bset);
-	if (!bounds)
-		goto error;
-	new_dim = bounds->n_row - 1;
-	bounds = isl_mat_left_hermite(bounds, 1, &U, NULL);
-	if (!bounds)
-		goto error;
-	U = isl_mat_drop_cols(U, 1 + new_dim, old_dim - new_dim);
-	bset = isl_basic_set_preimage(bset, isl_mat_copy(U));
-	if (!bset)
-		goto error;
-	*T = U;
-	isl_mat_free(bounds);
-	return bset;
-error:
-	isl_mat_free(bounds);
-	isl_mat_free(U);
-	isl_basic_set_free(bset);
-	return NULL;
-}
-
 /* Find a sample integer point, if any, in bset, which is known
  * to have equalities.  If bset contains no integer points, then
  * return a zero-length vector.
@@ -347,6 +207,105 @@ static struct isl_mat *initial_basis(struct isl_tab *tab)
 	return Q;
 }
 
+/* Compute the minimum of the current ("level") basis row over "tab"
+ * and store the result in position "level" of "min".
+ */
+static enum isl_lp_result compute_min(isl_ctx *ctx, struct isl_tab *tab,
+	__isl_keep isl_vec *min, int level)
+{
+	return isl_tab_min(tab, tab->basis->row[1 + level],
+			    ctx->one, &min->el[level], NULL, 0);
+}
+
+/* Compute the maximum of the current ("level") basis row over "tab"
+ * and store the result in position "level" of "max".
+ */
+static enum isl_lp_result compute_max(isl_ctx *ctx, struct isl_tab *tab,
+	__isl_keep isl_vec *max, int level)
+{
+	enum isl_lp_result res;
+	unsigned dim = tab->n_var;
+
+	isl_seq_neg(tab->basis->row[1 + level] + 1,
+		    tab->basis->row[1 + level] + 1, dim);
+	res = isl_tab_min(tab, tab->basis->row[1 + level],
+		    ctx->one, &max->el[level], NULL, 0);
+	isl_seq_neg(tab->basis->row[1 + level] + 1,
+		    tab->basis->row[1 + level] + 1, dim);
+	isl_int_neg(max->el[level], max->el[level]);
+
+	return res;
+}
+
+/* Perform a greedy search for an integer point in the set represented
+ * by "tab", given that the minimal rational value (rounded up to the
+ * nearest integer) at "level" is smaller than the maximal rational
+ * value (rounded down to the nearest integer).
+ *
+ * Return 1 if we have found an integer point (if tab->n_unbounded > 0
+ * then we may have only found integer values for the bounded dimensions
+ * and it is the responsibility of the caller to extend this solution
+ * to the unbounded dimensions).
+ * Return 0 if greedy search did not result in a solution.
+ * Return -1 if some error occurred.
+ *
+ * We assign a value half-way between the minimum and the maximum
+ * to the current dimension and check if the minimal value of the
+ * next dimension is still smaller than (or equal) to the maximal value.
+ * We continue this process until either
+ * - the minimal value (rounded up) is greater than the maximal value
+ *	(rounded down).  In this case, greedy search has failed.
+ * - we have exhausted all bounded dimensions, meaning that we have
+ *	found a solution.
+ * - the sample value of the tableau is integral.
+ * - some error has occurred.
+ */
+static int greedy_search(isl_ctx *ctx, struct isl_tab *tab,
+	__isl_keep isl_vec *min, __isl_keep isl_vec *max, int level)
+{
+	struct isl_tab_undo *snap;
+	enum isl_lp_result res;
+
+	snap = isl_tab_snap(tab);
+
+	do {
+		isl_int_add(tab->basis->row[1 + level][0],
+			    min->el[level], max->el[level]);
+		isl_int_fdiv_q_ui(tab->basis->row[1 + level][0],
+			    tab->basis->row[1 + level][0], 2);
+		isl_int_neg(tab->basis->row[1 + level][0],
+			    tab->basis->row[1 + level][0]);
+		if (isl_tab_add_valid_eq(tab, tab->basis->row[1 + level]) < 0)
+			return -1;
+		isl_int_set_si(tab->basis->row[1 + level][0], 0);
+
+		if (++level >= tab->n_var - tab->n_unbounded)
+			return 1;
+		if (isl_tab_sample_is_integer(tab))
+			return 1;
+
+		res = compute_min(ctx, tab, min, level);
+		if (res == isl_lp_error)
+			return -1;
+		if (res != isl_lp_ok)
+			isl_die(ctx, isl_error_internal,
+				"expecting bounded rational solution",
+				return -1);
+		res = compute_max(ctx, tab, max, level);
+		if (res == isl_lp_error)
+			return -1;
+		if (res != isl_lp_ok)
+			isl_die(ctx, isl_error_internal,
+				"expecting bounded rational solution",
+				return -1);
+	} while (isl_int_le(min->el[level], max->el[level]));
+
+	if (isl_tab_rollback(tab, snap) < 0)
+		return -1;
+
+	return 0;
+}
+
 /* Given a tableau representing a set, find and return
  * an integer point in the set, if there is any.
  *
@@ -386,6 +345,9 @@ static struct isl_mat *initial_basis(struct isl_tab *tab)
  * basis vector.  "init" is true if we want the first value at the current
  * level and false if we want the next value.
  *
+ * At the start of each level, we first check if we can find a solution
+ * using greedy search.  If not, we continue with the exhaustive search.
+ *
  * The initial basis is the identity matrix.  If the range in some direction
  * contains more than one integer value, we perform basis reduction based
  * on the value of ctx->opt->gbr
@@ -455,34 +417,38 @@ struct isl_vec *isl_tab_sample(struct isl_tab *tab)
 	reduced = 0;
 
 	while (level >= 0) {
-		int empty = 0;
 		if (init) {
-			res = isl_tab_min(tab, tab->basis->row[1 + level],
-				    ctx->one, &min->el[level], NULL, 0);
-			if (res == isl_lp_empty)
-				empty = 1;
-			isl_assert(ctx, res != isl_lp_unbounded, goto error);
+			int choice;
+
+			res = compute_min(ctx, tab, min, level);
 			if (res == isl_lp_error)
 				goto error;
-			if (!empty && isl_tab_sample_is_integer(tab))
+			if (res != isl_lp_ok)
+				isl_die(ctx, isl_error_internal,
+					"expecting bounded rational solution",
+					goto error);
+			if (isl_tab_sample_is_integer(tab))
 				break;
-			isl_seq_neg(tab->basis->row[1 + level] + 1,
-				    tab->basis->row[1 + level] + 1, dim);
-			res = isl_tab_min(tab, tab->basis->row[1 + level],
-				    ctx->one, &max->el[level], NULL, 0);
-			isl_seq_neg(tab->basis->row[1 + level] + 1,
-				    tab->basis->row[1 + level] + 1, dim);
-			isl_int_neg(max->el[level], max->el[level]);
-			if (res == isl_lp_empty)
-				empty = 1;
-			isl_assert(ctx, res != isl_lp_unbounded, goto error);
+			res = compute_max(ctx, tab, max, level);
 			if (res == isl_lp_error)
 				goto error;
-			if (!empty && isl_tab_sample_is_integer(tab))
+			if (res != isl_lp_ok)
+				isl_die(ctx, isl_error_internal,
+					"expecting bounded rational solution",
+					goto error);
+			if (isl_tab_sample_is_integer(tab))
 				break;
-			if (!empty && !reduced &&
-			    ctx->opt->gbr != ISL_GBR_NEVER &&
-			    isl_int_lt(min->el[level], max->el[level])) {
+			choice = isl_int_lt(min->el[level], max->el[level]);
+			if (choice) {
+				int g;
+				g = greedy_search(ctx, tab, min, max, level);
+				if (g < 0)
+					goto error;
+				if (g)
+					break;
+			}
+			if (!reduced && choice &&
+			    ctx->opt->gbr != ISL_GBR_NEVER) {
 				unsigned gbr_only_first;
 				if (ctx->opt->gbr == ISL_GBR_ONCE)
 					ctx->opt->gbr = ISL_GBR_NEVER;
@@ -502,7 +468,7 @@ struct isl_vec *isl_tab_sample(struct isl_tab *tab)
 		} else
 			isl_int_add_ui(min->el[level], min->el[level], 1);
 
-		if (empty || isl_int_gt(min->el[level], max->el[level])) {
+		if (isl_int_gt(min->el[level], max->el[level])) {
 			level--;
 			init = 0;
 			if (level >= 0)
@@ -1054,7 +1020,8 @@ static int tab_shift_cone(struct isl_tab *tab,
 		isl_vec_free(row);
 		if (isl_int_is_zero(v))
 			continue;
-		tab = isl_tab_extend(tab, 1);
+		if (isl_tab_extend_cons(tab, 1) < 0)
+			goto error;
 		isl_int_add(bset->ineq[i][0], bset->ineq[i][0], v);
 		ok = isl_tab_add_ineq(tab, bset->ineq[i]) >= 0;
 		isl_int_sub(bset->ineq[i][0], bset->ineq[i][0], v);
@@ -1148,27 +1115,6 @@ error:
 	return NULL;
 }
 
-static struct isl_vec *pip_sample(struct isl_basic_set *bset)
-{
-	struct isl_mat *T;
-	struct isl_ctx *ctx;
-	struct isl_vec *sample;
-
-	bset = isl_basic_set_skew_to_positive_orthant(bset, &T);
-	if (!bset)
-		return NULL;
-
-	ctx = bset->ctx;
-	sample = isl_pip_basic_set_sample(bset);
-
-	if (sample && sample->size != 0)
-		sample = isl_mat_vec_product(T, sample);
-	else
-		isl_mat_free(T);
-
-	return sample;
-}
-
 static struct isl_vec *basic_set_sample(struct isl_basic_set *bset, int bounded)
 {
 	struct isl_ctx *ctx;
@@ -1205,13 +1151,7 @@ static struct isl_vec *basic_set_sample(struct isl_basic_set *bset, int bounded)
 	if (dim == 1)
 		return interval_sample(bset);
 
-	switch (bset->ctx->opt->ilp_solver) {
-	case ISL_ILP_PIP:
-		return pip_sample(bset);
-	case ISL_ILP_GBR:
-		return bounded ? sample_bounded(bset) : gbr_sample(bset);
-	}
-	isl_assert(bset->ctx, 0, );
+	return bounded ? sample_bounded(bset) : gbr_sample(bset);
 error:
 	isl_basic_set_free(bset);
 	return NULL;
