@@ -11,9 +11,10 @@
 
 #include <isl_ctx_private.h>
 #include <isl_mat_private.h>
+#include <isl_vec_private.h>
 #include "isl_map_private.h"
 #include "isl_tab.h"
-#include <isl/seq.h>
+#include <isl_seq.h>
 #include <isl_config.h>
 
 /*
@@ -171,15 +172,6 @@ int isl_tab_extend_vars(struct isl_tab *tab, unsigned n_new)
 	return 0;
 }
 
-struct isl_tab *isl_tab_extend(struct isl_tab *tab, unsigned n_new)
-{
-	if (isl_tab_extend_cons(tab, n_new) >= 0)
-		return tab;
-
-	isl_tab_free(tab);
-	return NULL;
-}
-
 static void free_undo_record(struct isl_tab_undo *undo)
 {
 	switch (undo->type) {
@@ -869,8 +861,7 @@ error:
 	return NULL;
 }
 
-struct isl_tab *isl_tab_add_sample(struct isl_tab *tab,
-	__isl_take isl_vec *sample)
+int isl_tab_add_sample(struct isl_tab *tab, __isl_take isl_vec *sample)
 {
 	if (!tab || !sample)
 		goto error;
@@ -893,11 +884,10 @@ struct isl_tab *isl_tab_add_sample(struct isl_tab *tab,
 	tab->sample_index[tab->n_sample] = tab->n_sample;
 	tab->n_sample++;
 
-	return tab;
+	return 0;
 error:
 	isl_vec_free(sample);
-	isl_tab_free(tab);
-	return NULL;
+	return -1;
 }
 
 struct isl_tab *isl_tab_drop_sample(struct isl_tab *tab, int s)
@@ -1101,14 +1091,14 @@ int isl_tab_pivot(struct isl_tab *tab, int row, int col)
 	int i, j;
 	int sgn;
 	int t;
+	isl_ctx *ctx;
 	struct isl_mat *mat = tab->mat;
 	struct isl_tab_var *var;
 	unsigned off = 2 + tab->M;
 
-	if (tab->mat->ctx->abort) {
-		isl_ctx_set_error(tab->mat->ctx, isl_error_abort);
+	ctx = isl_tab_get_ctx(tab);
+	if (isl_ctx_next_operation(ctx) < 0)
 		return -1;
-	}
 
 	isl_int_swap(mat->row[row][0], mat->row[row][off + col]);
 	sgn = isl_int_sgn(mat->row[row][0]);
