@@ -14,10 +14,10 @@
           S6(c1, c3, c5, c7 - 1);
           S7(c1, c3, c5, c7);
         }
-        if (N >= 32 * c5 + 33) {
-          S6(c1, c3, c5, 32 * c5 + 31);
-        } else
+        if (32 * c5 + 31 >= N) {
           S6(c1, c3, c5, N - 1);
+        } else
+          S6(c1, c3, c5, 32 * c5 + 31);
       }
   S8();
 }
